@@ -27,7 +27,7 @@ class Expression:
     def __init__(self, **args):
         self.key = self.__class__.__name__.lower()
         self.args = args
-        self._set_parent(*args.values())
+        self._set_parent(**args)
         self.parent = None
         self.arg_key = None
 
@@ -87,13 +87,14 @@ class Expression:
             value: value to set the arg to.
         """
         self.args[arg] = value
-        self._set_parent(value)
+        self._set_parent(arg=value)
 
-    def _set_parent(self, *nodes):
-        for node in nodes:
+    def _set_parent(self, **kwargs):
+        for arg_key, node in kwargs.items():
             for v in ensure_list(node):
                 if isinstance(v, Expression):
                     v.parent = self
+                    v.arg_key = arg_key
 
     @property
     def depth(self):
@@ -104,7 +105,7 @@ class Expression:
             return self.parent.depth + 1
         return 0
 
-    def find(self, *expression_types):
+    def find(self, *expression_types, bfs=True):
         """
         Returns the first node in this tree which matches at least one of
         the specified types.
@@ -116,9 +117,9 @@ class Expression:
             the node which matches the criteria or None if no node matching
             the criteria was found.
         """
-        return next(self.find_all(*expression_types), None)
+        return next(self.find_all(*expression_types, bfs=bfs), None)
 
-    def find_all(self, *expression_types):
+    def find_all(self, *expression_types, bfs=True):
         """
         Returns a generator object which visits all nodes in this tree and only
         yields those that match at least one of the specified expression types.
@@ -129,10 +130,18 @@ class Expression:
         Returns:
             the generator object.
         """
-        for expression, _, _ in self.walk():
+        for expression, _, _ in self.walk(bfs=bfs):
             if isinstance(expression, expression_types):
                 yield expression
 
+    def find_ancestor(self, *expression_types):
+        ancestor = self.parent
+        while not isinstance(ancestor, expression_types):
+            ancestor = ancestor.parent
+            if not ancestor:
+                return None
+        return ancestor
+
     def walk(self, bfs=True):
         """
         Returns a generator object which visits all nodes in this tree.
@@ -147,9 +156,9 @@ class Expression:
         if bfs:
             yield from self.bfs()
         else:
-            yield from self.dfs(self.parent, None)
+            yield from self.dfs(self.parent)
 
-    def dfs(self, parent, key):
+    def dfs(self, parent=None, key=None, prune=None):
         """
         Returns a generator object which visits all nodes in this tree in
         the DFS (Depth-first) order.
@@ -158,17 +167,17 @@ class Expression:
             the generator object.
         """
         yield self, parent, key
+        if prune and prune(self, parent, key):
+            return
 
         for k, v in self.args.items():
             nodes = ensure_list(v)
 
             for node in nodes:
                 if isinstance(node, Expression):
-                    yield from node.dfs(self, k)
-                else:
-                    yield node, self, k
+                    yield from node.dfs(self, k, prune)
 
-    def bfs(self):
+    def bfs(self, prune=None):
         """
         Returns a generator object which visits all nodes in this tree in
         the BFS (Breadth-first) order.
@@ -182,13 +191,16 @@ class Expression:
             item, parent, key = queue.popleft()
 
             yield item, parent, key
+            if prune and prune(item, parent, key):
+                continue
 
             if isinstance(item, Expression):
                 for k, v in item.args.items():
                     nodes = ensure_list(v)
 
                     for node in nodes:
-                        queue.append((node, item, k))
+                        if isinstance(node, Expression):
+                            queue.append((node, item, k))
 
     def __repr__(self):
         return self.to_s()
@@ -253,7 +265,34 @@ class Expression:
         if not isinstance(new_node, Expression) or new_node is not node:
             return new_node
 
-        for k, v in new_node.args.items():
+        self._replace_children(
+            new_node, lambda child: child.transform(fun, *args, copy=False, **kwargs)
+        )
+        return new_node
+
+    def replace(self, *expressions):
+        """
+        Swap out this expression with a new expression.
+
+        For example::
+
+            >>> tree = Select().select("x").from_("tbl")
+            >>> tree.find(Column).replace(Column(this="y"))
+            >>> tree.sql()
+            'SELECT y FROM tbl'
+
+        Args:
+            expression (Expression): new node
+        """
+        if not self.parent:
+            return
+
+        self._replace_children(
+            self.parent, lambda child: expressions if child is self else child
+        )
+
+    def _replace_children(self, node, fun):
+        for k, v in node.args.items():
             is_list_arg = isinstance(v, list)
 
             child_nodes = v if is_list_arg else [v]
@@ -261,14 +300,14 @@ class Expression:
 
             for cn in child_nodes:
                 if isinstance(cn, Expression):
-                    new_child_node = cn.transform(fun, *args, copy=False, **kwargs)
-                    new_child_node.parent = new_node
+                    cns = ensure_list(fun(cn))
+                    for child_node in cns:
+                        child_node.parent = node
                 else:
-                    new_child_node = cn
-                new_child_nodes.append(new_child_node)
+                    cns = [cn]
+                new_child_nodes.extend(cns)
 
-            new_node.args[k] = new_child_nodes if is_list_arg else new_child_nodes[0]
-        return new_node
+            node.args[k] = new_child_nodes if is_list_arg else new_child_nodes[0]
 
     def assert_is(self, type_):
         """
@@ -341,6 +380,16 @@ class Condition(Expression):
         return not_(self)
 
 
+class DerivedTable:
+    @property
+    def alias_column_names(self):
+        table_alias = self.args.get("alias")  # pylint: disable=no-member
+        if not table_alias:
+            return []
+        column_list = table_alias.assert_is(TableAlias).args.get("columns") or []
+        return [c.name for c in column_list]
+
+
 class Annotation(Expression):
     arg_types = {
         "this": True,
@@ -392,7 +441,7 @@ class WithinGroup(Expression):
     arg_types = {"this": True, "expression": False}
 
 
-class CTE(Expression):
+class CTE(Expression, DerivedTable):
     arg_types = {"this": True, "alias": True}
 
 
@@ -579,13 +628,35 @@ class Subqueryable:
         instance = _maybe_copy(self, copy)
         return Subquery(
             this=instance,
-            alias=_to_identifier(alias),
+            alias=to_identifier(alias),
         )
 
+    @property
+    def ctes(self):
+        with_ = self.args.get("with")  # pylint: disable=no-member
+        if not with_:
+            return []
+        return with_.args.get("expressions", [])
+
 
 class Union(Subqueryable, Expression):
     arg_types = {"with": False, "this": True, "expression": True, "distinct": False}
 
+    @property
+    def named_selects(self):
+        named_selects = (
+            self.args["this"].named_selects + self.args["expression"].named_selects
+        )
+        return sorted(set(named_selects), key=named_selects.index)
+
+    @property
+    def left(self):
+        return self.this
+
+    @property
+    def right(self):
+        return self.args.get("expression")
+
 
 class Except(Union):
     pass
@@ -1133,8 +1204,16 @@ class Select(Subqueryable, Expression):
             ),
         )
 
+    @property
+    def named_selects(self):
+        return [e.alias_or_name for e in self.args["expressions"] if e.alias_or_name]
 
-class Subquery(Expression):
+    @property
+    def selects(self):
+        return self.args.get("expressions", [])
+
+
+class Subquery(Expression, DerivedTable):
     arg_types = {"this": True, "alias": False}
 
 
@@ -1279,6 +1358,10 @@ class EQ(Binary, Condition):
     pass
 
 
+class Escape(Binary):
+    pass
+
+
 class GT(Binary, Condition):
     pass
 
@@ -1531,10 +1614,22 @@ class DateStrToDate(Func):
     pass
 
 
+class DateToDateStr(Func):
+    pass
+
+
+class DateToDi(Func):
+    pass
+
+
 class Day(Func):
     pass
 
 
+class DiToDate(Func):
+    pass
+
+
 class Exp(Func):
     pass
 
@@ -1723,6 +1818,10 @@ class TsOrDsToDate(Func):
     pass
 
 
+class TsOrDiToDi(Func):
+    pass
+
+
 class UnixToStr(Func):
     arg_types = {"this": True, "format": True}
 
@@ -2045,7 +2144,7 @@ def not_(expression, dialect=None, **opts):
 SAFE_IDENTIFIER_RE = re.compile(r"^[a-zA-Z][\w]*$")
 
 
-def _to_identifier(alias, quoted=None):
+def to_identifier(alias, quoted=None):
     if alias is None:
         return None
     if isinstance(alias, Identifier):
@@ -2080,7 +2179,7 @@ def alias_(expression, alias, dialect=None, quoted=None, **opts):
         Alias: the aliased expression
     """
     exp = _maybe_parse(expression, dialect=dialect, parser_opts=opts)
-    alias = _to_identifier(alias, quoted=quoted)
+    alias = to_identifier(alias, quoted=quoted)
 
     if "alias" in exp.arg_types:
         exp = exp.copy()
@@ -2109,3 +2208,9 @@ def subquery(expression, alias=None, dialect=None, **opts):
 
     expression = _maybe_parse(expression, dialect=dialect, **opts).subquery(alias)
     return Select().from_(expression, dialect=dialect, **opts)
+
+
+TRUE = Boolean(this=True)
+FALSE = Boolean(this=False)
+NULL = Null()
+PREDICATES = (EQ, GT, GTE, ILike, In, Is, Like, LT, LTE, NEQ)
